/**
 * Copyright 2002, 2014 IBM Corp.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
 */
package clenotes;

import clenotes.Command;
import clenotes.ConfigurationSection;
import clenotes.Logger;
import clenotes.Option;
import com.google.common.base.Objects;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.Functions.Function0;

@SuppressWarnings("all")
public class Configuration {
  public final static String PROGRAM_NAME = "Command Line Email Client for IBM Notes";
  
  public final static String VERSION = "5.0.4";
  
  public final static String HEADER = new Function0<String>() {
    public String apply() {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append(Configuration.PROGRAM_NAME, "");
      _builder.append(" v");
      _builder.append(Configuration.VERSION, "");
      _builder.newLineIfNotEmpty();
      _builder.append("Copyright (C) 2002, 2014 by IBM Corporation.");
      _builder.newLine();
      _builder.append("Licensed under the Apache License v2.0.");
      _builder.newLine();
      return _builder.toString();
    }
  }.apply();
  
  public final static String MAILER = new Function0<String>() {
    public String apply() {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append(Configuration.PROGRAM_NAME, "");
      _builder.append(" v");
      _builder.append(Configuration.VERSION, "");
      return _builder.toString();
    }
  }.apply();
  
  public final static String LINE_SEPARATOR = new Function0<String>() {
    public String apply() {
      String _property = System.getProperty("line.separator");
      return _property;
    }
  }.apply();
  
  private static List<Option> globalOptions = new Function0<List<Option>>() {
    public List<Option> apply() {
      Vector<Option> _vector = new Vector<Option>();
      return _vector;
    }
  }.apply();
  
  public static Map<String,Option> globalOptionMap = new Function0<Map<String,Option>>() {
    public Map<String,Option> apply() {
      LinkedHashMap<String,Option> _newLinkedHashMap = CollectionLiterals.<String, Option>newLinkedHashMap();
      return _newLinkedHashMap;
    }
  }.apply();
  
  private static List<String> options = new Function0<List<String>>() {
    public List<String> apply() {
      Vector<String> _vector = new Vector<String>();
      return _vector;
    }
  }.apply();
  
  public static LinkedHashMap<Object,Object> commandMap = new Function0<LinkedHashMap<Object,Object>>() {
    public LinkedHashMap<Object,Object> apply() {
      LinkedHashMap<Object,Object> _newLinkedHashMap = CollectionLiterals.<Object, Object>newLinkedHashMap();
      return _newLinkedHashMap;
    }
  }.apply();
  
  private static List<String> commands = new Function0<List<String>>() {
    public List<String> apply() {
      Vector<String> _vector = new Vector<String>();
      return _vector;
    }
  }.apply();
  
  public static LinkedHashMap<Object,Object> configMap = new Function0<LinkedHashMap<Object,Object>>() {
    public LinkedHashMap<Object,Object> apply() {
      LinkedHashMap<Object,Object> _newLinkedHashMap = CollectionLiterals.<Object, Object>newLinkedHashMap();
      return _newLinkedHashMap;
    }
  }.apply();
  
  private static boolean LOG_ENABLED = false;
  
  private static boolean logParsing = false;
  
  public static void load() {
    try {
      File _file = new File("config/clenotes.cfg");
      FileReader _fileReader = new FileReader(_file);
      BufferedReader _bufferedReader = new BufferedReader(_fileReader);
      final BufferedReader br = _bufferedReader;
      String line = null;
      String _readLine = br.readLine();
      String _line = line = _readLine;
      boolean _notEquals = (!Objects.equal(_line, null));
      boolean _while = _notEquals;
      while (_while) {
        boolean _matched = false;
        if (!_matched) {
          boolean _or = false;
          boolean _startsWith = line.startsWith("#");
          if (_startsWith) {
            _or = true;
          } else {
            boolean _isEmpty = line.isEmpty();
            _or = (_startsWith || _isEmpty);
          }
          if (_or) {
            _matched=true;
          }
        }
        if (!_matched) {
          boolean _equals = line.equals("[options]");
          if (_equals) {
            _matched=true;
            ConfigurationSection.setOption();
          }
        }
        if (!_matched) {
          boolean _equals_1 = line.equals("[commands]");
          if (_equals_1) {
            _matched=true;
            ConfigurationSection.setCommand();
          }
        }
        if (!_matched) {
          boolean _equals_2 = line.equals("[configuration]");
          if (_equals_2) {
            _matched=true;
            ConfigurationSection.setConfiguration();
          }
        }
        if (!_matched) {
          boolean _isOption = ConfigurationSection.isOption();
          if (_isOption) {
            _matched=true;
            Configuration.options.add(line);
          }
        }
        if (!_matched) {
          boolean _isCommand = ConfigurationSection.isCommand();
          if (_isCommand) {
            _matched=true;
            Configuration.commands.add(line);
          }
        }
        if (!_matched) {
          boolean _isConfiguration = ConfigurationSection.isConfiguration();
          if (_isConfiguration) {
            _matched=true;
            boolean _contains = line.contains("write_log");
            if (_contains) {
              boolean _and = false;
              boolean _equals_3 = (Configuration.LOG_ENABLED == false);
              if (!_equals_3) {
                _and = false;
              } else {
                String _lowerCase = line.toLowerCase();
                boolean _contains_1 = _lowerCase.contains("true");
                _and = (_equals_3 && _contains_1);
              }
              if (_and) {
                Configuration.LOG_ENABLED = true;
              }
            }
            final String[] configEntry = line.split(":");
            String _get = configEntry[0];
            String _trim = _get.trim();
            String _get_1 = configEntry[1];
            String _trim_1 = _get_1.trim();
            Configuration.configMap.put(_trim, _trim_1);
          }
        }
        String _readLine_1 = br.readLine();
        String _line_1 = line = _readLine_1;
        boolean _notEquals_1 = (!Objects.equal(_line_1, null));
        _while = _notEquals_1;
      }
      br.close();
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static void printConfigFileEntries() {
    if (Configuration.logParsing) {
      String _plus = ("Options: " + Configuration.globalOptions);
      Logger.log(_plus);
      String _plus_1 = ("Commands: " + Configuration.commandMap);
      Logger.log(_plus_1);
      String _plus_2 = ("Configuration: " + Configuration.configMap);
      Logger.log(_plus_2);
    }
  }
  
  public static void parse() {
    for (final String option : Configuration.options) {
      {
        final int firstIndex = option.indexOf(":");
        String name = option.substring(0, firstIndex);
        int _plus = (firstIndex + 1);
        String desc = option.substring(_plus);
        String _replace = name.replace("--", "");
        String _trim = _replace.trim();
        name = _trim;
        boolean mandatoryValue = false;
        boolean _contains = name.contains("!");
        if (_contains) {
          mandatoryValue = true;
          String _replace_1 = name.replace("!", "");
          name = _replace_1;
        }
        String _removeFunctionName = Configuration.removeFunctionName(desc);
        desc = _removeFunctionName;
        Option _option = new Option();
        Option opt = _option;
        opt.setName(name);
        opt.setDescription(desc);
        opt.setMandatoryValue(mandatoryValue);
        Configuration.globalOptions.add(opt);
        String _name = opt.getName();
        Configuration.globalOptionMap.put(_name, opt);
      }
    }
    for (final String command : Configuration.commands) {
      {
        int _indexOf = command.indexOf(".");
        final String commandName = command.substring(0, _indexOf);
        boolean _contains = command.contains(".cmd");
        if (_contains) {
          int _indexOf_1 = command.indexOf(":");
          int _plus = (_indexOf_1 + 1);
          String commandDescription = command.substring(_plus);
          String _removeFunctionName = Configuration.removeFunctionName(commandDescription);
          commandDescription = _removeFunctionName;
          Command _command = new Command();
          Command cmd = _command;
          cmd.setName(commandName);
          cmd.setDescription(commandDescription);
          Configuration.commandMap.put(commandName, cmd);
        } else {
          Object _get = Configuration.commandMap.get(commandName);
          Command commandObj = ((Command) _get);
          boolean _contains_1 = command.contains("--");
          if (_contains_1) {
            int _indexOf_2 = command.indexOf(".");
            int _plus_1 = (_indexOf_2 + 1);
            String optionString = command.substring(_plus_1);
            if (Configuration.logParsing) {
              String _plus_2 = ("Option in config: " + optionString);
              Logger.log(_plus_2);
            }
            final int cInd = optionString.indexOf(":");
            String optionName = optionString.substring(0, cInd);
            final boolean mandatoryValue = optionName.contains("!");
            String _replace = optionName.replace("--", "");
            String _replace_1 = _replace.replace("!", "");
            String _trim = _replace_1.trim();
            optionName = _trim;
            if (Configuration.logParsing) {
              String _plus_3 = ("Option in config: " + optionName);
              Logger.log(_plus_3);
            }
            int _plus_4 = (cInd + 1);
            String _substring = optionString.substring(_plus_4);
            String optionDesc = _substring.trim();
            if (Configuration.logParsing) {
              String _plus_5 = ("Option desc config: " + optionDesc);
              Logger.log(_plus_5);
            }
            Option _option = new Option();
            Option opt = _option;
            opt.setMandatoryValue(mandatoryValue);
            opt.setName(optionName);
            opt.setDescription(optionDesc);
            Map<String,Option> cmdOpts = commandObj.getOptionMap();
            String _name = opt.getName();
            cmdOpts.put(_name, opt);
          }
        }
      }
    }
    boolean _isLogEnabled = Configuration.isLogEnabled();
    if (_isLogEnabled) {
      Set<Object> _keySet = Configuration.commandMap.keySet();
      for (final Object commandName : _keySet) {
        if (Configuration.logParsing) {
          Object _get = Configuration.commandMap.get(commandName);
          String _plus = ("Command object: " + _get);
          Logger.log(_plus);
        }
      }
    }
  }
  
  private static String removeFunctionName(final String _desc) {
    String desc = _desc;
    String _trim = desc.trim();
    desc = _trim;
    final int ind = desc.indexOf("|");
    int _minus = (-1);
    boolean _greaterThan = (ind > _minus);
    if (_greaterThan) {
      String _substring = desc.substring(0, ind);
      desc = _substring;
    }
    return desc;
  }
  
  public static boolean setLogEnabled(final boolean enabled) {
    boolean _LOG_ENABLED = Configuration.LOG_ENABLED = enabled;
    return _LOG_ENABLED;
  }
  
  public static boolean isLogEnabled() {
    return Configuration.LOG_ENABLED;
  }
  
  public static Object getConfig(final String configName) {
    return Configuration.configMap.get(configName);
  }
}
