/**
 * Copyright 2002, 2014 IBM Corp.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
 */
package clenotes.commands;

import clenotes.Command;
import clenotes.CommandLineArguments;
import clenotes.Configuration;
import clenotes.utils.DominoUtils;
import clenotes.utils.Input;
import clenotes.utils.Output;
import com.google.common.base.Objects;
import com.google.common.collect.Lists;
import java.util.Collections;
import lotus.domino.Database;
import lotus.domino.DateTime;
import lotus.domino.Document;
import lotus.domino.DocumentCollection;
import lotus.domino.Name;
import lotus.domino.Session;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;

@SuppressWarnings("all")
public class ReadMail {
  public static boolean readLatestMail(final Command cmd, final Session notesSession) {
    try {
      boolean _xblockexpression = false;
      {
        Database mailDb = DominoUtils.getMailDatabase(notesSession);
        DateTime dt = notesSession.createDateTime("Yesterday");
        String adjustDay = CommandLineArguments.getValue("adjust-day");
        boolean _notEquals = (!Objects.equal(adjustDay, null));
        if (_notEquals) {
          int _parseInt = Integer.parseInt(adjustDay);
          int _minus = (-_parseInt);
          dt.adjustDay(_minus);
        }
        DocumentCollection docCollection = mailDb.search("@All", dt);
        Document doc = DominoUtils.getLatestMailDocument(docCollection);
        boolean _readMail = ReadMail.readMail(doc, cmd, notesSession);
        _xblockexpression = (_readMail);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static boolean readMail(final Document mailDocument, final Command cmd, final Session session) {
    try {
      boolean _hasOption = cmd.hasOption("dxl");
      if (_hasOption) {
        String _generateXML = mailDocument.generateXML();
        InputOutput.<String>println(_generateXML);
        return false;
      }
      boolean _hasOption_1 = cmd.hasOption("fields");
      if (_hasOption_1) {
        DominoUtils.printFields(mailDocument);
        return false;
      }
      boolean _hasOption_2 = cmd.hasOption("fieldvalues");
      if (_hasOption_2) {
        String _optionValue = cmd.getOptionValue("fieldvalues");
        DominoUtils.printFieldValue(mailDocument, _optionValue);
        return false;
      }
      boolean printMailToScreen = true;
      boolean _hasOption_3 = cmd.hasOption("reply");
      if (_hasOption_3) {
        boolean replyToAll = false;
        boolean _hasOption_4 = cmd.hasOption("all");
        if (_hasOption_4) {
          replyToAll = true;
        }
        Document replyDoc = mailDocument.createReplyMessage(replyToAll);
        String _itemValueString = mailDocument.getItemValueString("Subject");
        String _plus = ("Re: " + _itemValueString);
        replyDoc.replaceItemValue("Subject", _plus);
        Name _userNameObject = session.getUserNameObject();
        String _addr821 = _userNameObject.getAddr821();
        replyDoc.replaceItemValue("From", _addr821);
        String body = DominoUtils.getMailBody(mailDocument);
        String replyBody = "";
        boolean _hasOption_5 = cmd.hasOption("body");
        if (_hasOption_5) {
          String _optionValue_1 = cmd.getOptionValue("body");
          replyBody = _optionValue_1;
        } else {
          String confirmation = Input.prompt("No reply body. Reply without body (yes/no)? ");
          String _lowerCase = confirmation.toLowerCase();
          String _trim = _lowerCase.trim();
          confirmation = _trim;
          boolean _notEquals = (!Objects.equal(confirmation, "yes"));
          if (_notEquals) {
            InputOutput.<String>println("Use --body  to specify reply body.");
            return false;
          }
        }
        String _plus_1 = (replyBody + "\n\n==================================================\n\n");
        String _plus_2 = (_plus_1 + body);
        replyBody = _plus_2;
        replyDoc.replaceItemValue("Body", replyBody);
        replyDoc.replaceItemValue("$Mailer", Configuration.MAILER);
        replyDoc.setSaveMessageOnSend(true);
        String _itemValueString_1 = mailDocument.getItemValueString("From");
        replyDoc.send(_itemValueString_1);
        InputOutput.<String>println("Mail replied.");
        printMailToScreen = false;
      }
      boolean _or = false;
      boolean _hasOption_6 = cmd.hasOption("detach-file");
      if (_hasOption_6) {
        _or = true;
      } else {
        boolean _hasOption_7 = cmd.hasOption("detach-all");
        _or = (_hasOption_6 || _hasOption_7);
      }
      if (_or) {
        String dir = null;
        boolean _hasOption_8 = cmd.hasOption("detach-dir");
        if (_hasOption_8) {
          String _optionValue_2 = cmd.getOptionValue("detach-dir");
          dir = _optionValue_2;
        }
        String attachmentToBeSaved = null;
        boolean _hasOption_9 = cmd.hasOption("detach-file");
        if (_hasOption_9) {
          String _optionValue_3 = cmd.getOptionValue("detach-file");
          attachmentToBeSaved = _optionValue_3;
        }
        boolean detachAllFiles = false;
        boolean _hasOption_10 = cmd.hasOption("detach-all");
        if (_hasOption_10) {
          detachAllFiles = true;
        }
        DominoUtils.getAttachmentNames(mailDocument, dir, attachmentToBeSaved, detachAllFiles, session);
        printMailToScreen = false;
      }
      boolean isDeleted = false;
      String confirmation_1 = "no";
      boolean _hasOption_11 = cmd.hasOption("delete");
      if (_hasOption_11) {
        boolean _hasOption_12 = cmd.hasOption("no-confirmation");
        if (_hasOption_12) {
          confirmation_1 = "yes";
        } else {
          String _itemValueString_2 = mailDocument.getItemValueString("From");
          String _itemValueString_3 = mailDocument.getItemValueString("Subject");
          String prompt = String.format("Delete mail from \'%s\' with subject \'%s\' (yes/no)? ", _itemValueString_2, _itemValueString_3);
          String _prompt = Input.prompt(prompt);
          confirmation_1 = _prompt;
          String _lowerCase_1 = confirmation_1.toLowerCase();
          String _trim_1 = _lowerCase_1.trim();
          confirmation_1 = _trim_1;
        }
        boolean _equals = Objects.equal(confirmation_1, "yes");
        if (_equals) {
          isDeleted = true;
          mailDocument.remove(false);
          InputOutput.<String>println("Mail deleted.");
        } else {
          InputOutput.<String>println("Mail not deleted.");
        }
        printMailToScreen = false;
      }
      boolean _hasOption_13 = cmd.hasOption("move-to-folder");
      if (_hasOption_13) {
        String targetFolder = cmd.getOptionValue("move-to-folder");
        String sourceFolder = cmd.getOptionValue("folder");
        mailDocument.putInFolder(targetFolder);
        boolean _notEquals_1 = (!Objects.equal(sourceFolder, null));
        if (_notEquals_1) {
          mailDocument.removeFromFolder(sourceFolder);
        }
        String _plus_3 = ("Mail moved to folder " + targetFolder);
        String _plus_4 = (_plus_3 + ".");
        InputOutput.<String>println(_plus_4);
        return false;
      }
      boolean _not = (!printMailToScreen);
      if (_not) {
        return isDeleted;
      }
      Object deliveredDate = DominoUtils.getItemValue(mailDocument, "DeliveredDate");
      InputOutput.println();
      int columnWidth = 20;
      int lineStartIndentLevel = 0;
      String _string = deliveredDate.toString();
      Output.prettyPrintln(Collections.<String>unmodifiableList(Lists.<String>newArrayList("Date :", _string)), columnWidth, lineStartIndentLevel);
      String _itemValueString_4 = mailDocument.getItemValueString("From");
      Output.prettyPrintln(Collections.<String>unmodifiableList(Lists.<String>newArrayList("From :", _itemValueString_4)), columnWidth, lineStartIndentLevel);
      String _itemValues = DominoUtils.getItemValues(mailDocument, "SendTo");
      Output.prettyPrintln(Collections.<String>unmodifiableList(Lists.<String>newArrayList("To :", _itemValues)), columnWidth, lineStartIndentLevel);
      String _itemValueString_5 = mailDocument.getItemValueString("ReplyTo");
      Output.prettyPrintln(Collections.<String>unmodifiableList(Lists.<String>newArrayList("Reply-To :", _itemValueString_5)), columnWidth, lineStartIndentLevel);
      String _itemValues_1 = DominoUtils.getItemValues(mailDocument, "CopyTo");
      Output.prettyPrintln(Collections.<String>unmodifiableList(Lists.<String>newArrayList("Copy-To :", _itemValues_1)), columnWidth, lineStartIndentLevel);
      String importance = DominoUtils.importance(mailDocument);
      boolean _notEquals_2 = (!Objects.equal(importance, null));
      if (_notEquals_2) {
        Output.prettyPrintln(Collections.<String>unmodifiableList(Lists.<String>newArrayList("Importance :", importance)), columnWidth, lineStartIndentLevel);
      }
      String _itemValueString_6 = mailDocument.getItemValueString("Subject");
      Output.prettyPrintln(Collections.<String>unmodifiableList(Lists.<String>newArrayList("Subject :", _itemValueString_6)), columnWidth, lineStartIndentLevel);
      String _attachmentNames = DominoUtils.getAttachmentNames(mailDocument, session);
      Output.prettyPrintln(Collections.<String>unmodifiableList(Lists.<String>newArrayList("Attachments :", ((String) _attachmentNames))), columnWidth, lineStartIndentLevel);
      boolean printBody = true;
      boolean _hasOption_14 = cmd.hasOption("no-body");
      if (_hasOption_14) {
        printBody = false;
      }
      if (printBody) {
        Output.prettyPrintln(Collections.<String>unmodifiableList(Lists.<String>newArrayList("Body :")), columnWidth, lineStartIndentLevel);
        String body_1 = DominoUtils.getMailBody(mailDocument);
        InputOutput.<String>println(body_1);
      }
      return isDeleted;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
