/**
 * Copyright 2002, 2014 IBM Corp.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
 */
package clenotes.commands;

import clenotes.Command;
import clenotes.Configuration;
import clenotes.Logger;
import clenotes.utils.DominoUtils;
import com.google.common.base.Objects;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import lotus.domino.Database;
import lotus.domino.Document;
import lotus.domino.EmbeddedObject;
import lotus.domino.MIMEEntity;
import lotus.domino.MIMEHeader;
import lotus.domino.RichTextItem;
import lotus.domino.Session;
import lotus.domino.Stream;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IterableExtensions;

@SuppressWarnings("all")
public class Send {
  public static int execute(final Command cmd, final Session notesSession) {
    try {
      Database mailDb = DominoUtils.getMailDatabase(notesSession);
      Document newMailDocument = mailDb.createDocument();
      String sender = notesSession.getUserName();
      String principal = cmd.getOptionValue("principal");
      boolean _notEquals = (!Objects.equal(principal, null));
      if (_notEquals) {
        newMailDocument.replaceItemValue("Principal", principal);
      }
      String subject = cmd.getOptionValue("subject");
      List<String> toList = cmd.getOptionValues("to");
      String toFile = cmd.getOptionValue("file-to");
      boolean _and = false;
      boolean _equals = Objects.equal(toList, null);
      if (!_equals) {
        _and = false;
      } else {
        boolean _equals_1 = Objects.equal(toFile, null);
        _and = (_equals && _equals_1);
      }
      if (_and) {
        InputOutput.<String>println("Recipient not specified. Use --to or --option to set recipient.");
        InputOutput.<String>println("Mail not sent.");
        return (-102);
      }
      Vector<String> _vector = new Vector<String>();
      Vector<String> toVector = _vector;
      Vector ccVector = null;
      Vector bccVector = null;
      boolean _notEquals_1 = (!Objects.equal(toList, null));
      if (_notEquals_1) {
        for (final String to : toList) {
          String _trim = to.trim();
          toVector.add(_trim);
        }
      }
      boolean _notEquals_2 = (!Objects.equal(toFile, null));
      if (_notEquals_2) {
        Vector<String> _readRecipientFile = DominoUtils.readRecipientFile(toFile);
        toVector = _readRecipientFile;
      }
      List<String> ccList = cmd.getOptionValues("cc");
      boolean _notEquals_3 = (!Objects.equal(ccList, null));
      if (_notEquals_3) {
        Vector<Object> _vector_1 = new Vector<Object>();
        ccVector = _vector_1;
        for (final String cc : ccList) {
          String _trim_1 = cc.trim();
          ccVector.add(_trim_1);
        }
      }
      String ccFile = cmd.getOptionValue("file-cc");
      boolean _notEquals_4 = (!Objects.equal(ccFile, null));
      if (_notEquals_4) {
        Vector<String> _readRecipientFile_1 = DominoUtils.readRecipientFile(ccFile);
        ccVector = _readRecipientFile_1;
      }
      List<String> bccList = cmd.getOptionValues("bcc");
      boolean _notEquals_5 = (!Objects.equal(bccList, null));
      if (_notEquals_5) {
        Vector<Object> _vector_2 = new Vector<Object>();
        bccVector = _vector_2;
        for (final String bcc : bccList) {
          String _trim_2 = bcc.trim();
          bccVector.add(_trim_2);
        }
      }
      String bccFile = cmd.getOptionValue("file-bcc");
      boolean _notEquals_6 = (!Objects.equal(bccFile, null));
      if (_notEquals_6) {
        Vector<String> _readRecipientFile_2 = DominoUtils.readRecipientFile(bccFile);
        bccVector = _readRecipientFile_2;
      }
      String _plus = ("To  list: " + toVector);
      Logger.log(_plus);
      String _plus_1 = ("Cc  list: " + ccVector);
      Logger.log(_plus_1);
      String _plus_2 = ("Bcc list: " + bccVector);
      Logger.log(_plus_2);
      newMailDocument.replaceItemValue("Form", "Memo");
      newMailDocument.replaceItemValue("From", sender);
      newMailDocument.replaceItemValue("SendTo", toVector);
      boolean _notEquals_7 = (!Objects.equal(ccList, null));
      if (_notEquals_7) {
        newMailDocument.replaceItemValue("CopyTo", ccVector);
      }
      boolean _notEquals_8 = (!Objects.equal(bccList, null));
      if (_notEquals_8) {
        newMailDocument.replaceItemValue("BlindCopyTo", bccVector);
      }
      String replyTo = cmd.getOptionValue("replyto");
      boolean _notEquals_9 = (!Objects.equal(replyTo, null));
      if (_notEquals_9) {
        newMailDocument.replaceItemValue("ReplyTo", replyTo);
      }
      newMailDocument.replaceItemValue("Subject", subject);
      newMailDocument.replaceItemValue("$Mailer", Configuration.MAILER);
      boolean _hasOption = cmd.hasOption("urgent");
      if (_hasOption) {
        newMailDocument.replaceItemValue("Importance", "1");
      }
      boolean _hasOption_1 = cmd.hasOption("encrypt");
      if (_hasOption_1) {
        newMailDocument.setEncryptOnSend(true);
      }
      boolean _hasOption_2 = cmd.hasOption("sign");
      if (_hasOption_2) {
        newMailDocument.setSignOnSend(true);
      }
      String body = cmd.getOptionValue("body");
      boolean _notEquals_10 = (!Objects.equal(body, null));
      if (_notEquals_10) {
        String _replace = body.replace("\\n", "\n");
        body = _replace;
      }
      boolean _isLogEnabled = Configuration.isLogEnabled();
      if (_isLogEnabled) {
        Charset _defaultCharset = Charset.defaultCharset();
        String _name = _defaultCharset.name();
        String _plus_3 = ("default charset: " + _name);
        Logger.log(_plus_3);
        Map<String,Charset> availableCharsets = Charset.availableCharsets();
        Logger.log("Available charsets:");
        Set<String> _keySet = availableCharsets.keySet();
        for (final String charset : _keySet) {
          Charset _get = availableCharsets.get(charset);
          String _name_1 = _get.name();
          String _plus_4 = ("  " + _name_1);
          Logger.log(_plus_4);
        }
      }
      String bodyFile = cmd.getOptionValue("file-body");
      boolean _notEquals_11 = (!Objects.equal(bodyFile, null));
      if (_notEquals_11) {
        Charset charset_1 = Charset.forName("UTF-8");
        String _name_2 = charset_1.name();
        String _plus_5 = ("Reading body file using charset: " + _name_2);
        Logger.log(_plus_5);
        FileInputStream _fileInputStream = new FileInputStream(bodyFile);
        InputStreamReader _inputStreamReader = new InputStreamReader(_fileInputStream, charset_1);
        BufferedReader _bufferedReader = new BufferedReader(_inputStreamReader);
        BufferedReader f = _bufferedReader;
        ArrayList<Object> bodyLines = CollectionLiterals.<Object>newArrayList();
        String line = f.readLine();
        boolean _notEquals_12 = (!Objects.equal(line, null));
        boolean _while = _notEquals_12;
        while (_while) {
          {
            String _plus_6 = (line + "\n");
            bodyLines.add(_plus_6);
            String _readLine = f.readLine();
            line = _readLine;
          }
          boolean _notEquals_13 = (!Objects.equal(line, null));
          _while = _notEquals_13;
        }
        f.close();
        boolean _equals_2 = Objects.equal(body, null);
        if (_equals_2) {
          String _join = IterableExtensions.join(bodyLines, "");
          body = _join;
        } else {
          String _plus_6 = (body + "\n\n");
          String _join_1 = IterableExtensions.join(bodyLines, "");
          String _plus_7 = (_plus_6 + _join_1);
          body = _plus_7;
        }
      }
      boolean _hasOption_3 = cmd.hasOption("html");
      if (_hasOption_3) {
        Stream stream = notesSession.createStream();
        notesSession.setConvertMIME(false);
        MIMEEntity mimeBody = newMailDocument.createMIMEEntity();
        MIMEHeader header = mimeBody.createHeader("Content-Type");
        header.setHeaderVal("multipart/mixed");
        MIMEEntity bodyEntity = mimeBody.createChildEntity();
        stream.writeText(body);
        bodyEntity.setContentFromText(stream, "text/html;charset=\"UTF-8\"", MIMEEntity.ENC_NONE);
        String[] sigList = Send.getSignature(cmd);
        String signature = sigList[0];
        String signatureFile = sigList[1];
        boolean _notEquals_13 = (!Objects.equal(signature, null));
        if (_notEquals_13) {
          MIMEEntity signatureEntity = mimeBody.createChildEntity();
          Stream _createStream = notesSession.createStream();
          stream = _createStream;
          stream.writeText(signature);
          signatureEntity.setContentFromText(stream, "text/html;charset=\"UTF-8\"", MIMEEntity.ENC_NONE);
        }
        boolean _notEquals_14 = (!Objects.equal(signatureFile, null));
        if (_notEquals_14) {
          MIMEEntity signatureEntity_1 = mimeBody.createChildEntity();
          Stream _createStream_1 = notesSession.createStream();
          stream = _createStream_1;
          stream.writeText(signatureFile);
          signatureEntity_1.setContentFromText(stream, "text/html;charset=\"UTF-8\"", MIMEEntity.ENC_NONE);
        }
        String[] attachmentList = Send.getFilesToAttach(cmd);
        for (final String attachment : attachmentList) {
          {
            MIMEEntity attachmentEntity = mimeBody.createChildEntity();
            MIMEHeader _header = attachmentEntity.createHeader("Content-Disposition");
            String _plus_8 = ("attachment;filename=" + attachment);
            _header.setHeaderVal(_plus_8);
            Stream _createStream_2 = notesSession.createStream();
            stream = _createStream_2;
            FileInputStream _fileInputStream_1 = new FileInputStream(attachment);
            FileInputStream fileInput = _fileInputStream_1;
            stream.setContents(fileInput);
            attachmentEntity.setContentFromBytes(stream, "application/octet-stream", MIMEEntity.ENC_IDENTITY_BINARY);
            fileInput.close();
          }
        }
        notesSession.setConvertMIME(true);
      } else {
        boolean _notEquals_15 = (!Objects.equal(body, null));
        if (_notEquals_15) {
          RichTextItem richTextBody = newMailDocument.createRichTextItem("Body");
          richTextBody.appendText(body);
          String[] sigList_1 = Send.getSignature(cmd);
          String signature_1 = sigList_1[0];
          String signatureFile_1 = sigList_1[1];
          boolean _notEquals_16 = (!Objects.equal(signature_1, null));
          if (_notEquals_16) {
            String _plus_8 = ("\n\n" + signature_1);
            richTextBody.appendText(_plus_8);
          }
          boolean _notEquals_17 = (!Objects.equal(signatureFile_1, null));
          if (_notEquals_17) {
            String _plus_9 = ("\n\n" + signatureFile_1);
            richTextBody.appendText(_plus_9);
          }
          String[] attachmentList_1 = Send.getFilesToAttach(cmd);
          for (final String attachment_1 : attachmentList_1) {
            {
              File _file = new File(attachment_1);
              File file = _file;
              String _absolutePath = file.getAbsolutePath();
              richTextBody.embedObject(EmbeddedObject.EMBED_ATTACHMENT, null, _absolutePath, null);
            }
          }
        }
      }
      boolean _hasOption_4 = cmd.hasOption("nosave");
      boolean _not = (!_hasOption_4);
      if (_not) {
        newMailDocument.save();
      }
      boolean _hasOption_5 = cmd.hasOption("html");
      if (_hasOption_5) {
        newMailDocument.send(false);
      } else {
        newMailDocument.send();
      }
      String toStr = toVector.get(0);
      String andOthers = "";
      int _size = toVector.size();
      boolean _greaterThan = (_size > 1);
      if (_greaterThan) {
        andOthers = " and others";
      }
      String _plus_10 = ("Mail sent to " + toStr);
      String _plus_11 = (_plus_10 + andOthers);
      String _plus_12 = (_plus_11 + " with subject: ");
      String _plus_13 = (_plus_12 + subject);
      InputOutput.<String>println(_plus_13);
      return 0;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static String[] getFilesToAttach(final Command cmd) {
    String[] _xblockexpression = null;
    {
      String attachments = cmd.getOptionValue("attach");
      String[] attachmentList = ((String[])Conversions.unwrapArray(CollectionLiterals.<String>newArrayList(), String.class));
      boolean _notEquals = (!Objects.equal(attachments, null));
      if (_notEquals) {
        String[] _split = attachments.split(",");
        attachmentList = _split;
      }
      _xblockexpression = (attachmentList);
    }
    return _xblockexpression;
  }
  
  public static String[] getSignature(final Command cmd) {
    try {
      String signature = null;
      String signatureFile = null;
      boolean _hasOption = cmd.hasOption("signature");
      if (_hasOption) {
        String _optionValue = cmd.getOptionValue("signature");
        signature = _optionValue;
        String _replace = signature.replace("\\n", "\n");
        signature = _replace;
      }
      boolean _hasOption_1 = cmd.hasOption("file-signature");
      if (_hasOption_1) {
        String _signatureFile = cmd.getOptionValue("file-signature");
        FileInputStream _fileInputStream = new FileInputStream(_signatureFile);
        InputStreamReader _inputStreamReader = new InputStreamReader(_fileInputStream);
        BufferedReader _bufferedReader = new BufferedReader(_inputStreamReader);
        BufferedReader f = _bufferedReader;
        ArrayList<Object> lines = CollectionLiterals.<Object>newArrayList();
        String line = f.readLine();
        boolean _notEquals = (!Objects.equal(line, null));
        boolean _while = _notEquals;
        while (_while) {
          {
            String _plus = (line + "\n");
            lines.add(_plus);
            String _readLine = f.readLine();
            line = _readLine;
          }
          boolean _notEquals_1 = (!Objects.equal(line, null));
          _while = _notEquals_1;
        }
        f.close();
        String _join = IterableExtensions.join(lines, "");
        signatureFile = _join;
      }
      String[] returnValue = new String[2];
      returnValue[0] = signature;
      returnValue[1] = signatureFile;
      return returnValue;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
