/**
 * Copyright 2002, 2014 IBM Corp.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
 */
package clenotes.commands;

import clenotes.Command;
import clenotes.CommandLineArguments;
import clenotes.Logger;
import clenotes.commands.ReadMail;
import clenotes.utils.DominoUtils;
import com.google.common.base.Objects;
import java.util.List;
import lotus.domino.Database;
import lotus.domino.DateTime;
import lotus.domino.Document;
import lotus.domino.DocumentCollection;
import lotus.domino.Session;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;

@SuppressWarnings("all")
public class Today {
  public static int execute(final Command cmd, final Session notesSession) {
    try {
      Database mailDb = DominoUtils.getMailDatabase(notesSession);
      DateTime dt = notesSession.createDateTime("Yesterday");
      String adjustDay = CommandLineArguments.getValue("adjust-day");
      boolean _notEquals = (!Objects.equal(adjustDay, null));
      if (_notEquals) {
        int _parseInt = Integer.parseInt(adjustDay);
        int _minus = (-_parseInt);
        dt.adjustDay(_minus);
      }
      DocumentCollection docCollection = null;
      boolean _hasOption = cmd.hasOption("all");
      if (_hasOption) {
        DocumentCollection _search = mailDb.search("@All", dt);
        docCollection = _search;
      } else {
        String _userName = notesSession.getUserName();
        String _plus = ("From!=\"" + _userName);
        String _plus_1 = (_plus + "\" & (Form=\"Memo\" | Form=\"Reply\")");
        DocumentCollection _search_1 = mailDb.search(_plus_1, dt);
        docCollection = _search_1;
      }
      DocumentCollection _sortDocumentCollectionIfNecessary = DominoUtils.sortDocumentCollectionIfNecessary(docCollection, mailDb, cmd);
      docCollection = _sortDocumentCollectionIfNecessary;
      String readIndex = cmd.getOptionValue("read");
      boolean _and = false;
      boolean _notEquals_1 = (!Objects.equal(readIndex, null));
      if (!_notEquals_1) {
        _and = false;
      } else {
        boolean _equals = readIndex.equals("*");
        _and = (_notEquals_1 && _equals);
      }
      if (_and) {
        int n = 1;
        Document doc = docCollection.getNthDocument(n);
        boolean _notEquals_2 = (!Objects.equal(doc, null));
        boolean _while = _notEquals_2;
        while (_while) {
          {
            ReadMail.readMail(doc, cmd, notesSession);
            int _plus_2 = (n + 1);
            n = _plus_2;
            doc.recycle();
            Document _nthDocument = docCollection.getNthDocument(n);
            doc = _nthDocument;
          }
          boolean _notEquals_3 = (!Objects.equal(doc, null));
          _while = _notEquals_3;
        }
      } else {
        int _readIndex = (-1);
        boolean _notEquals_3 = (!Objects.equal(readIndex, null));
        if (_notEquals_3) {
          int _parseInt_1 = Integer.parseInt(readIndex);
          _readIndex = _parseInt_1;
        }
        List<? extends Object> values = DominoUtils.getMails(docCollection, cmd, _readIndex, false, notesSession);
        Object numberOfMails = values.get(0);
        Object _get = values.get(1);
        String outputTxt = ((String) _get);
        Object _get_1 = values.get(2);
        Document mailDoc = ((Document) _get_1);
        boolean _equals_1 = Objects.equal(mailDoc, null);
        if (_equals_1) {
          String txt = "mails";
          boolean _equals_2 = Objects.equal(numberOfMails, Integer.valueOf(1));
          if (_equals_2) {
            txt = "mail";
          }
          String adjustedDay = "";
          boolean _notEquals_4 = (!Objects.equal(adjustDay, null));
          if (_notEquals_4) {
            String _plus_2 = ("-" + adjustDay);
            adjustedDay = _plus_2;
          }
          String _plus_3 = (numberOfMails + " ");
          String _plus_4 = (_plus_3 + txt);
          String _plus_5 = (_plus_4 + " since yesterday");
          String _plus_6 = (_plus_5 + adjustedDay);
          String _plus_7 = (_plus_6 + ".");
          InputOutput.<String>println(_plus_7);
          InputOutput.println();
          InputOutput.<String>println(outputTxt);
        } else {
          String _string = mailDoc.toString();
          String _plus_8 = ("Reading mail... " + _string);
          Logger.log(_plus_8);
          ReadMail.readMail(mailDoc, cmd, notesSession);
        }
      }
      return 0;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
