/**
 * Copyright 2002, 2014 IBM Corp.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
 */
package clenotes.utils;

import clenotes.Command;
import clenotes.CommandLineArguments;
import clenotes.Configuration;
import clenotes.Logger;
import clenotes.utils.Output;
import com.google.common.base.Objects;
import com.google.common.collect.Lists;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import lotus.domino.Database;
import lotus.domino.DateTime;
import lotus.domino.DbDirectory;
import lotus.domino.Document;
import lotus.domino.DocumentCollection;
import lotus.domino.EmbeddedObject;
import lotus.domino.Item;
import lotus.domino.MIMEEntity;
import lotus.domino.MIMEHeader;
import lotus.domino.RichTextItem;
import lotus.domino.Session;
import lotus.domino.Stream;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.StringExtensions;

@SuppressWarnings("all")
public class DominoUtils {
  public static Database getMailDatabase(final Session notesSession) {
    try {
      DbDirectory dbDirectory = null;
      String serverName = CommandLineArguments.getValue("server-name");
      DbDirectory _dbDirectory = notesSession.getDbDirectory(serverName);
      dbDirectory = _dbDirectory;
      String replicaId = CommandLineArguments.getValue("replica-id");
      String databaseName = CommandLineArguments.getValue("database-name");
      return DominoUtils.openDatabase(dbDirectory, databaseName, replicaId);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static Database openDatabase(final DbDirectory dbDirectory, final String databaseName, final String _replicaId) {
    try {
      Database _xblockexpression = null;
      {
        String replicaId = _replicaId;
        boolean _and = false;
        boolean _equals = Objects.equal(databaseName, null);
        if (!_equals) {
          _and = false;
        } else {
          boolean _equals_1 = Objects.equal(replicaId, null);
          _and = (_equals && _equals_1);
        }
        if (_and) {
          boolean _hasOption = CommandLineArguments.hasOption("local");
          if (_hasOption) {
            Database db = dbDirectory.openMailDatabase();
            String _replicaID = db.getReplicaID();
            replicaId = _replicaID;
            db.recycle();
          } else {
            try {
              return dbDirectory.openMailDatabase();
            } catch (Throwable _e) {
              throw Exceptions.sneakyThrow(_e);
            }
          }
        }
        boolean _notEquals = (!Objects.equal(replicaId, null));
        if (_notEquals) {
          try {
            return dbDirectory.openDatabaseByReplicaID(replicaId);
          } catch (Throwable _e_1) {
            throw Exceptions.sneakyThrow(_e_1);
          }
        }
        Database _xifexpression = null;
        boolean _notEquals_1 = (!Objects.equal(databaseName, null));
        if (_notEquals_1) {
          try {
            return dbDirectory.openDatabase(databaseName, false);
          } catch (Throwable _e_2) {
            throw Exceptions.sneakyThrow(_e_2);
          }
        }
        _xblockexpression = (_xifexpression);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static Object getItemValue(final Document doc, final String itemName) {
    try {
      Object _xblockexpression = null;
      {
        Vector _itemValue = doc.getItemValue(itemName);
        Vector<? extends Object> value = ((Vector<?>) _itemValue);
        Object rValue = null;
        boolean _and = false;
        boolean _notEquals = (!Objects.equal(value, null));
        if (!_notEquals) {
          _and = false;
        } else {
          int _size = value.size();
          boolean _greaterThan = (_size > 0);
          _and = (_notEquals && _greaterThan);
        }
        if (_and) {
          Object _firstElement = value.firstElement();
          rValue = _firstElement;
        } else {
          boolean _equals = Objects.equal(itemName, "DeliveredDate");
          if (_equals) {
            Vector _itemValue_1 = doc.getItemValue("PostedDate");
            value = ((Vector<?>) _itemValue_1);
            boolean _and_1 = false;
            boolean _notEquals_1 = (!Objects.equal(value, null));
            if (!_notEquals_1) {
              _and_1 = false;
            } else {
              int _size_1 = value.size();
              boolean _greaterThan_1 = (_size_1 > 0);
              _and_1 = (_notEquals_1 && _greaterThan_1);
            }
            if (_and_1) {
              Object _firstElement_1 = value.firstElement();
              rValue = _firstElement_1;
            }
          }
        }
        _xblockexpression = (rValue);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static String getItemValues(final Document doc, final String itemName) {
    try {
      Vector _itemValue = doc.getItemValue(itemName);
      Vector<? extends Object> value = ((Vector<?>) _itemValue);
      StringBuffer _stringBuffer = new StringBuffer();
      StringBuffer str = _stringBuffer;
      for (final Object v : value) {
        {
          str.append(v);
          str.append(", ");
        }
      }
      String values = str.toString();
      int _length = values.length();
      boolean _greaterEqualsThan = (_length >= 2);
      if (_greaterEqualsThan) {
        int _length_1 = values.length();
        int _minus = (_length_1 - 2);
        String _substring = values.substring(0, _minus);
        values = _substring;
      }
      return values;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static DocumentCollection sortDocumentCollectionIfNecessary(final DocumentCollection docCollection, final Database database, final Command cmd) {
    try {
      String sortOrder = cmd.getOptionValue("sortorder");
      String sortField = cmd.getOptionValue("sortfield");
      List<String> allowedOrders = Collections.<String>unmodifiableList(Lists.<String>newArrayList("ASC", "DESC"));
      List<String> allowedFields = Collections.<String>unmodifiableList(Lists.<String>newArrayList("DATE", "SUBJECT", "FROM"));
      boolean _or = false;
      boolean _notEquals = (!Objects.equal(sortOrder, null));
      if (_notEquals) {
        _or = true;
      } else {
        boolean _notEquals_1 = (!Objects.equal(sortField, null));
        _or = (_notEquals || _notEquals_1);
      }
      if (_or) {
        boolean _equals = Objects.equal(sortOrder, null);
        if (_equals) {
          String _get = allowedOrders.get(0);
          sortOrder = _get;
        }
        boolean _equals_1 = Objects.equal(sortField, null);
        if (_equals_1) {
          String _get_1 = allowedFields.get(0);
          sortField = _get_1;
        }
        boolean _contains = allowedOrders.contains(sortOrder);
        boolean _not = (!_contains);
        if (_not) {
          String _join = IterableExtensions.join(allowedOrders, ",");
          String _plus = ("Sort order not recognized. Allowed sort orders: " + _join);
          IllegalArgumentException _illegalArgumentException = new IllegalArgumentException(_plus);
          throw _illegalArgumentException;
        }
        boolean _contains_1 = allowedFields.contains(sortField);
        boolean _not_1 = (!_contains_1);
        if (_not_1) {
          String _join_1 = IterableExtensions.join(allowedFields, ",");
          String _plus_1 = ("Sort field not recognized. Allowed sort orders: " + _join_1);
          IllegalArgumentException _illegalArgumentException_1 = new IllegalArgumentException(_plus_1);
          throw _illegalArgumentException_1;
        }
        DocumentCollection newDocCollection = database.getProfileDocCollection("PROFILE_THAT_SHOULD_STILL_NOT_EXIST");
        String _plus_2 = ("Sort by " + sortField);
        Logger.log(_plus_2);
        LinkedHashMap<String,String> docsToBeSorted = CollectionLiterals.<String, String>newLinkedHashMap();
        int n = 1;
        Document doc = docCollection.getNthDocument(n);
        boolean _notEquals_2 = (!Objects.equal(doc, null));
        boolean _while = _notEquals_2;
        while (_while) {
          {
            String field = "N/A";
            String docId = doc.getUniversalID();
            boolean _matched = false;
            if (!_matched) {
              if (Objects.equal(sortField,"DATE")) {
                _matched=true;
                Object _itemValue = DominoUtils.getItemValue(doc, "DeliveredDate");
                DateTime deliveredDate = ((DateTime) _itemValue);
                boolean _notEquals_3 = (!Objects.equal(deliveredDate, null));
                if (_notEquals_3) {
                  Date _javaDate = deliveredDate.toJavaDate();
                  long _time = _javaDate.getTime();
                  String _string = Long.valueOf(_time).toString();
                  field = _string;
                }
              }
            }
            if (!_matched) {
              if (Objects.equal(sortField,"SUBJECT")) {
                _matched=true;
                String subject = doc.getItemValueString("Subject");
                boolean _and = false;
                boolean _notEquals_4 = (!Objects.equal(subject, null));
                if (!_notEquals_4) {
                  _and = false;
                } else {
                  int _length = subject.length();
                  boolean _greaterThan = (_length > 0);
                  _and = (_notEquals_4 && _greaterThan);
                }
                if (_and) {
                  field = subject;
                }
              }
            }
            if (!_matched) {
              if (Objects.equal(sortField,"FROM")) {
                _matched=true;
                String fromAddr = doc.getItemValueString("From");
                String _plus_3 = ("FROM: " + fromAddr);
                Logger.log(_plus_3);
                boolean _and_1 = false;
                boolean _notEquals_5 = (!Objects.equal(fromAddr, null));
                if (!_notEquals_5) {
                  _and_1 = false;
                } else {
                  int _length_1 = fromAddr.length();
                  boolean _greaterThan_1 = (_length_1 > 0);
                  _and_1 = (_notEquals_5 && _greaterThan_1);
                }
                if (_and_1) {
                  field = fromAddr;
                }
              }
            }
            boolean _containsKey = docsToBeSorted.containsKey(field);
            if (_containsKey) {
              String old = docsToBeSorted.get(field);
              String _plus_4 = (old + " ");
              String _plus_5 = (_plus_4 + docId);
              docsToBeSorted.put(field, _plus_5);
            } else {
              docsToBeSorted.put(field, docId);
            }
            int _plus_6 = (n + 1);
            n = _plus_6;
            doc.recycle();
            Document _nthDocument = docCollection.getNthDocument(n);
            doc = _nthDocument;
          }
          boolean _notEquals_3 = (!Objects.equal(doc, null));
          _while = _notEquals_3;
        }
        List<String> sortedDocsFields = DominoUtils.sortMap(docsToBeSorted);
        boolean _equals_2 = sortOrder.equals("DESC");
        if (_equals_2) {
          ListExtensions.<String>reverse(sortedDocsFields);
        }
        for (final String _field : sortedDocsFields) {
          {
            String id = docsToBeSorted.get(_field);
            boolean _contains_2 = id.contains(" ");
            if (_contains_2) {
              String[] idList = id.split(" ");
              for (final String _id : idList) {
                Document _documentByUNID = database.getDocumentByUNID(_id);
                newDocCollection.addDocument(_documentByUNID);
              }
            } else {
              Document _documentByUNID_1 = database.getDocumentByUNID(id);
              newDocCollection.addDocument(_documentByUNID_1);
            }
          }
        }
        return newDocCollection;
      }
      return docCollection;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static DocumentCollection getMailDocCollection(final DocumentCollection docCollection, final Database database, final Command cmd) {
    try {
      int listStartIndex = (-1);
      int listEndIndex = 10000;
      boolean hasRangeIndex = false;
      boolean _hasOption = cmd.hasOption("start");
      if (_hasOption) {
        String _optionValue = cmd.getOptionValue("start");
        Integer _valueOf = Integer.valueOf(_optionValue);
        listStartIndex = (_valueOf).intValue();
        hasRangeIndex = true;
      }
      boolean _hasOption_1 = cmd.hasOption("end");
      if (_hasOption_1) {
        String _optionValue_1 = cmd.getOptionValue("end");
        Integer _valueOf_1 = Integer.valueOf(_optionValue_1);
        listEndIndex = (_valueOf_1).intValue();
        hasRangeIndex = true;
      }
      boolean _not = (!hasRangeIndex);
      if (_not) {
        return docCollection;
      }
      int index = 1;
      DocumentCollection newDocCollection = database.getProfileDocCollection("PROFILE_THAT_SHOULD_STILL_NOT_EXIST");
      Document doc = docCollection.getNthDocument(index);
      boolean _notEquals = (!Objects.equal(doc, null));
      boolean _while = _notEquals;
      while (_while) {
        {
          boolean _and = false;
          boolean _greaterEqualsThan = (index >= listStartIndex);
          if (!_greaterEqualsThan) {
            _and = false;
          } else {
            boolean _lessEqualsThan = (index <= listEndIndex);
            _and = (_greaterEqualsThan && _lessEqualsThan);
          }
          if (_and) {
            newDocCollection.addDocument(doc);
          }
          int _plus = (index + 1);
          index = _plus;
          Document _nthDocument = docCollection.getNthDocument(index);
          doc = _nthDocument;
        }
        boolean _notEquals_1 = (!Objects.equal(doc, null));
        _while = _notEquals_1;
      }
      return newDocCollection;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static List<String> sortMap(final Map<? extends Object,? extends Object> map) {
    List<String> _xblockexpression = null;
    {
      Set<? extends Object> keySet = map.keySet();
      ArrayList<String> vr = CollectionLiterals.<String>newArrayList();
      for (final Object key : keySet) {
        vr.add(((String) key));
      }
      List<String> _sort = IterableExtensions.<String>sort(vr);
      _xblockexpression = (_sort);
    }
    return _xblockexpression;
  }
  
  public static String getMailBody(final Document mailDocument) {
    String _xblockexpression = null;
    {
      String body = DominoUtils.getMailBody(mailDocument, "Body");
      boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(body);
      if (_isNullOrEmpty) {
        String _mailBody = DominoUtils.getMailBody(mailDocument, "$Body");
        body = _mailBody;
      }
      String _removeMultipleEmptyLines = DominoUtils.removeMultipleEmptyLines(body);
      body = _removeMultipleEmptyLines;
      _xblockexpression = (body);
    }
    return _xblockexpression;
  }
  
  public static String getMailBody(final Document mailDocument, final String fieldName) {
    try {
      String _xblockexpression = null;
      {
        Vector bodyItems = mailDocument.getItemValue(fieldName);
        String body = DominoUtils.concatToString(bodyItems);
        boolean _equals = Objects.equal(body, "");
        if (_equals) {
          Item item = mailDocument.getFirstItem(fieldName);
          boolean _notEquals = (!Objects.equal(item, null));
          if (_notEquals) {
            Class<? extends Item> _class = item.getClass();
            String _name = _class.getName();
            String _plus = ("Getting body from: " + _name);
            Logger.log(_plus);
          }
          boolean _matched = false;
          if (!_matched) {
            if ((item instanceof RichTextItem)) {
              _matched=true;
              RichTextItem rtitem = ((RichTextItem) item);
              String _text = rtitem.getText();
              body = _text;
            }
          }
          if (!_matched) {
            if ((item instanceof Item)) {
              _matched=true;
              Item _item = ((Item) item);
              String _text_1 = _item.getText();
              body = _text_1;
              boolean _equals_1 = Objects.equal(body, null);
              if (_equals_1) {
                MIMEEntity mimeEntity = _item.getMIMEEntity();
                String _text_2 = DominoUtils.getText(mimeEntity);
                body = _text_2;
              }
            }
          }
          if (!_matched) {
            if (Objects.equal(item,null)) {
              _matched=true;
              body = "[Unable to get mail body]";
            }
          }
        }
        _xblockexpression = (body);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static String concatToString(final Vector<? extends Object> items) {
    String _xblockexpression = null;
    {
      StringBuffer _stringBuffer = new StringBuffer();
      StringBuffer txt = _stringBuffer;
      for (final Object item : items) {
        {
          String _string = item.toString();
          txt.append(_string);
          txt.append(Configuration.LINE_SEPARATOR);
        }
      }
      String _string = txt.toString();
      _xblockexpression = (_string);
    }
    return _xblockexpression;
  }
  
  public static String importance(final Document mailDocument) {
    try {
      String _xblockexpression = null;
      {
        String importance = mailDocument.getItemValueString("Importance");
        boolean _equals = Objects.equal(importance, "1");
        if (_equals) {
          return "High";
        }
        boolean _equals_1 = Objects.equal(importance, "2");
        if (_equals_1) {
          return "Normal";
        }
        _xblockexpression = (null);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static String getAttachmentNames(final Document doc, final Session session) {
    return DominoUtils.getAttachmentNames(doc, null, null, false, session);
  }
  
  public static String getAttachmentNames(final Document doc, final String detachDir, final String detachAttachmentName, final boolean detachAll, final Session session) {
    try {
      boolean _or = false;
      if (detachAll) {
        _or = true;
      } else {
        boolean _notEquals = (!Objects.equal(detachAttachmentName, null));
        _or = (detachAll || _notEquals);
      }
      if (_or) {
        InputOutput.<String>println("Saving attachments from...");
        String _formattedMail = DominoUtils.getFormattedMail(doc, "!dtfs", 0, " ", session);
        String _plus = ("   " + _formattedMail);
        InputOutput.<String>println(_plus);
      }
      String _plus_1 = ("getAttachmentNames: " + doc);
      Logger.log(_plus_1);
      boolean hasNativeMIME = doc.hasItem("$NoteHasNativeMIME");
      boolean _and = false;
      if (!hasNativeMIME) {
        _and = false;
      } else {
        boolean _hasEmbedded = doc.hasEmbedded();
        _and = (hasNativeMIME && _hasEmbedded);
      }
      boolean hasMIMEAttachments = _and;
      boolean _and_1 = false;
      if (!hasMIMEAttachments) {
        _and_1 = false;
      } else {
        boolean _or_1 = false;
        if (detachAll) {
          _or_1 = true;
        } else {
          boolean _notEquals_1 = (!Objects.equal(detachAttachmentName, null));
          _or_1 = (detachAll || _notEquals_1);
        }
        _and_1 = (hasMIMEAttachments && _or_1);
      }
      if (_and_1) {
        String names = "";
        String _detachMIMEAttachments = DominoUtils.detachMIMEAttachments(doc, session, detachDir, false, detachAttachmentName);
        names = _detachMIMEAttachments;
        return names;
      }
      if (hasNativeMIME) {
        return DominoUtils.detachMIMEAttachments(doc, session, detachDir, true, detachAttachmentName);
      }
      if (hasNativeMIME) {
        MIMEEntity mimeEntity = doc.getMIMEEntity();
        String _plus_2 = ("Mime entity: " + mimeEntity);
        Logger.log(_plus_2);
      }
      Item body = doc.getFirstItem("Body");
      boolean _equals = Objects.equal(body, null);
      if (_equals) {
        Item _firstItem = doc.getFirstItem("$Body");
        body = _firstItem;
      }
      Vector<Object> _vector = new Vector<Object>();
      Vector<Object> docAttachments = _vector;
      StringBuffer _stringBuffer = new StringBuffer();
      StringBuffer str = _stringBuffer;
      boolean someError = false;
      boolean _equals_1 = Objects.equal(body, null);
      if (_equals_1) {
        Logger.log("Body is null");
        Item _firstItem_1 = doc.getFirstItem("Subject");
        String _plus_3 = ("  Subject: " + _firstItem_1);
        Logger.log(_plus_3);
        String _universalID = doc.getUniversalID();
        String _plus_4 = ("  UNID   : " + _universalID);
        Logger.log(_plus_4);
      } else {
        Class<? extends Item> _class = body.getClass();
        String _name = _class.getName();
        String _plus_5 = ("Body item class: " + _name);
        Logger.log(_plus_5);
        try {
          Vector _embeddedObjects = doc.getEmbeddedObjects();
          docAttachments = _embeddedObjects;
          int _size = docAttachments.size();
          String _plus_6 = ("Attachments size from doc: " + Integer.valueOf(_size));
          Logger.log(_plus_6);
        } catch (final Throwable _t) {
          if (_t instanceof Exception) {
            final Exception e = (Exception)_t;
            someError = true;
            str.append("[ERROR] Exception when getting attachments");
          } else {
            throw Exceptions.sneakyThrow(_t);
          }
        }
        int _size_1 = docAttachments.size();
        boolean _equals_2 = (_size_1 == 0);
        if (_equals_2) {
          try {
            RichTextItem rtitem = ((RichTextItem) body);
            Vector _embeddedObjects_1 = rtitem.getEmbeddedObjects();
            docAttachments = _embeddedObjects_1;
            int _size_2 = docAttachments.size();
            String _plus_7 = ("Attachments size from RichTextItem: " + Integer.valueOf(_size_2));
            Logger.log(_plus_7);
          } catch (final Throwable _t_1) {
            if (_t_1 instanceof Exception) {
              final Exception e_1 = (Exception)_t_1;
              Logger.log(e_1);
            } else {
              throw Exceptions.sneakyThrow(_t_1);
            }
          }
        }
        int _size_3 = docAttachments.size();
        boolean _equals_3 = (_size_3 == 0);
        if (_equals_3) {
          Logger.log("Check attachments from all items..");
          Vector items = doc.getItems();
          for (final Object _item : items) {
            {
              Item item = ((Item) _item);
              if ((item instanceof RichTextItem)) {
                RichTextItem rtitem_1 = ((RichTextItem) item);
                Vector eObjects = rtitem_1.getEmbeddedObjects();
                for (final Object eo : eObjects) {
                  docAttachments.add(eo);
                }
              } else {
                int _type = item.getType();
                boolean _equals_4 = (_type == Item.ATTACHMENT);
                if (_equals_4) {
                  Logger.log("File attachment found");
                  boolean _hasEmbedded_1 = doc.hasEmbedded();
                  String _plus_8 = ("  Doc has embedded: " + Boolean.valueOf(_hasEmbedded_1));
                  Logger.log(_plus_8);
                  Vector eObjects_1 = doc.getEmbeddedObjects();
                  for (final Object eo_1 : eObjects_1) {
                    docAttachments.add(eo_1);
                  }
                  int _size_4 = eObjects_1.size();
                  String _plus_9 = ("  Tota embedded: " + Integer.valueOf(_size_4));
                  Logger.log(_plus_9);
                  Class<? extends Item> _class_1 = item.getClass();
                  String _name_1 = _class_1.getName();
                  String _plus_10 = ("  class: " + _name_1);
                  Logger.log(_plus_10);
                  String _name_2 = item.getName();
                  String _plus_11 = ("  name: " + _name_2);
                  Logger.log(_plus_11);
                  String _text = item.getText();
                  String _plus_12 = ("  text: " + _text);
                  Logger.log(_plus_12);
                  int _valueLength = item.getValueLength();
                  String _plus_13 = ("  value length: " + Integer.valueOf(_valueLength));
                  Logger.log(_plus_13);
                  String _valueString = item.getValueString();
                  String _trim = _valueString.trim();
                  String _plus_14 = ("  valuestring: " + _trim);
                  Logger.log(_plus_14);
                }
              }
            }
          }
        }
      }
      for (final Object o : docAttachments) {
        {
          EmbeddedObject a = ((EmbeddedObject) o);
          String _name_1 = a.getName();
          String _plus_8 = ("EmbeddedObject: " + _name_1);
          Logger.log(_plus_8);
          int _type = a.getType();
          boolean _equals_4 = (_type == EmbeddedObject.EMBED_ATTACHMENT);
          if (_equals_4) {
            String fileName = a.getName();
            str.append(fileName);
            str.append(", ");
            String dir = ".";
            boolean _or_2 = false;
            if (detachAll) {
              _or_2 = true;
            } else {
              boolean _notEquals_2 = (!Objects.equal(detachAttachmentName, null));
              _or_2 = (detachAll || _notEquals_2);
            }
            if (_or_2) {
              boolean _notEquals_3 = (!Objects.equal(detachDir, null));
              if (_notEquals_3) {
                String _plus_9 = ("detach-dir: " + dir);
                Logger.log(_plus_9);
                String _replace = detachDir.replace("\\", "/");
                dir = _replace;
                boolean _endsWith = dir.endsWith("/");
                boolean _not = (!_endsWith);
                if (_not) {
                  String _plus_10 = (dir + "/");
                  dir = _plus_10;
                }
                String _plus_11 = ("detach-dir modified: " + dir);
                Logger.log(_plus_11);
              } else {
                dir = "./";
              }
              String extractedFile = (dir + fileName);
              boolean _or_3 = false;
              if (detachAll) {
                _or_3 = true;
              } else {
                boolean _equals_5 = Objects.equal(fileName, detachAttachmentName);
                _or_3 = (detachAll || _equals_5);
              }
              if (_or_3) {
                a.extractFile(extractedFile);
                String _plus_12 = ("   Attachment saved: " + extractedFile);
                String _plus_13 = (_plus_12 + ".");
                InputOutput.<String>println(_plus_13);
              }
            }
          }
        }
      }
      String names_1 = str.toString();
      boolean _endsWith = names_1.endsWith(",");
      if (_endsWith) {
        int _length = names_1.length();
        int _minus = (_length - 1);
        String _substring = names_1.substring(0, _minus);
        names_1 = _substring;
      }
      boolean _and_2 = false;
      int _length_1 = names_1.length();
      boolean _equals_4 = (_length_1 == 0);
      if (!_equals_4) {
        _and_2 = false;
      } else {
        _and_2 = (_equals_4 && hasMIMEAttachments);
      }
      if (_and_2) {
        names_1 = "[MIME attachments not yet implemented]";
      }
      return names_1;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static List<? extends Object> getMails(final DocumentCollection documentCollection, final Command cmd, final int docReadIndex, final boolean noRange, final Session session) {
    try {
      List<? extends Object> _xblockexpression = null;
      {
        String outputFormatString = CommandLineArguments.getValue("output-format");
        String delimiter = CommandLineArguments.getValue("delim");
        boolean _hasOption = CommandLineArguments.hasOption("tab");
        if (_hasOption) {
          delimiter = "\t";
        }
        String _plus = ("Doc read index: " + Integer.valueOf(docReadIndex));
        Logger.log(_plus);
        Document mailDocToBeRead = null;
        Document doc = documentCollection.getFirstDocument();
        int index = 0;
        int listStartIndex = (-1);
        int listEndIndex = 10000;
        boolean hasRangeIndex = false;
        boolean _not = (!noRange);
        if (_not) {
          boolean _hasOption_1 = cmd.hasOption("start");
          if (_hasOption_1) {
            String _optionValue = cmd.getOptionValue("start");
            Integer _valueOf = Integer.valueOf(_optionValue);
            listStartIndex = (_valueOf).intValue();
            hasRangeIndex = true;
          }
          boolean _hasOption_2 = cmd.hasOption("end");
          if (_hasOption_2) {
            String _optionValue_1 = cmd.getOptionValue("end");
            Integer _valueOf_1 = Integer.valueOf(_optionValue_1);
            listEndIndex = (_valueOf_1).intValue();
            hasRangeIndex = true;
          }
        }
        StringBuffer _stringBuffer = new StringBuffer();
        StringBuffer outputTxt = _stringBuffer;
        boolean isReceivedMail = false;
        boolean processDoc = false;
        boolean _notEquals = (!Objects.equal(doc, null));
        boolean _while = _notEquals;
        while (_while) {
          {
            isReceivedMail = true;
            boolean _equals = (docReadIndex == index);
            if (_equals) {
              mailDocToBeRead = doc;
            }
            processDoc = false;
            Object deliveredDate = DominoUtils.getItemValue(doc, "DeliveredDate");
            boolean _equals_1 = Objects.equal(deliveredDate, null);
            if (_equals_1) {
              isReceivedMail = false;
            }
            if (isReceivedMail) {
              processDoc = true;
              if (hasRangeIndex) {
                boolean _and = false;
                boolean _greaterEqualsThan = (index >= listStartIndex);
                if (!_greaterEqualsThan) {
                  _and = false;
                } else {
                  boolean _lessEqualsThan = (index <= listEndIndex);
                  _and = (_greaterEqualsThan && _lessEqualsThan);
                }
                if (_and) {
                  processDoc = true;
                } else {
                  processDoc = false;
                }
              }
              String _plus_1 = ("index: " + Integer.valueOf(index));
              String _plus_2 = (_plus_1 + " Prosessdoc: ");
              String _plus_3 = (_plus_2 + Boolean.valueOf(processDoc));
              String _plus_4 = (_plus_3 + ", start: ");
              String _plus_5 = (_plus_4 + Integer.valueOf(listStartIndex));
              String _plus_6 = (_plus_5 + ", end: ");
              String _plus_7 = (_plus_6 + Integer.valueOf(listEndIndex));
              Logger.log(_plus_7);
              if (processDoc) {
                boolean _notEquals_1 = (!Objects.equal(outputFormatString, null));
                if (_notEquals_1) {
                  String _formattedMail = DominoUtils.getFormattedMail(doc, outputFormatString, index, delimiter, session);
                  outputTxt.append(_formattedMail);
                } else {
                  String _plus_8 = ("[" + Integer.valueOf(index));
                  String _plus_9 = (_plus_8 + "] ");
                  outputTxt.append(_plus_9);
                  outputTxt.append(deliveredDate);
                  outputTxt.append(Configuration.LINE_SEPARATOR);
                  outputTxt.append("  ");
                  boolean _isSentByAgent = doc.isSentByAgent();
                  if (_isSentByAgent) {
                    outputTxt.append("From (Sent by agent?, probable sender): ");
                  } else {
                    outputTxt.append("From: ");
                  }
                  String fromField = doc.getItemValueString("From");
                  String principalField = doc.getItemValueString("Principal");
                  boolean _and_1 = false;
                  boolean _notEquals_2 = (!Objects.equal(principalField, null));
                  if (!_notEquals_2) {
                    _and_1 = false;
                  } else {
                    boolean _notEquals_3 = (!Objects.equal(principalField, ""));
                    _and_1 = (_notEquals_2 && _notEquals_3);
                  }
                  if (_and_1) {
                    boolean _notEquals_4 = (!Objects.equal(fromField, principalField));
                    if (_notEquals_4) {
                      String _trim = principalField.trim();
                      principalField = _trim;
                      boolean _notEquals_5 = (!Objects.equal(principalField, ""));
                      if (_notEquals_5) {
                        String _plus_10 = (principalField + " (");
                        String _plus_11 = (_plus_10 + fromField);
                        String _plus_12 = (_plus_11 + ")");
                        fromField = _plus_12;
                      }
                    }
                  }
                  outputTxt.append(fromField);
                  outputTxt.append(Configuration.LINE_SEPARATOR);
                  outputTxt.append("  ");
                  String subject = doc.getItemValueString("Subject");
                  outputTxt.append("Subject: ");
                  outputTxt.append(subject);
                  outputTxt.append(Configuration.LINE_SEPARATOR);
                  outputTxt.append("  ");
                  String importance = DominoUtils.importance(doc);
                  boolean _equals_2 = Objects.equal(importance, "High");
                  if (_equals_2) {
                    outputTxt.append("High Importance");
                    outputTxt.append(Configuration.LINE_SEPARATOR);
                    outputTxt.append("  ");
                  }
                  outputTxt.append("Attachments: ");
                  String _attachmentNames = DominoUtils.getAttachmentNames(doc, session);
                  outputTxt.append(_attachmentNames);
                  outputTxt.append(Configuration.LINE_SEPARATOR);
                  outputTxt.append("  ");
                }
                outputTxt.append(Configuration.LINE_SEPARATOR);
              }
              int _plus_13 = (index + 1);
              index = _plus_13;
            }
            Document _nextDocument = documentCollection.getNextDocument();
            doc = _nextDocument;
          }
          boolean _notEquals_1 = (!Objects.equal(doc, null));
          _while = _notEquals_1;
        }
        String mailTxt = outputTxt.toString();
        _xblockexpression = (Collections.<Object>unmodifiableList(Lists.<Object>newArrayList(Integer.valueOf(index), mailTxt, mailDocToBeRead)));
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static String getFormattedMail(final Document mailDoc, final String _formatString, final int index, final String _delimiter, final Session session) {
    try {
      String delimiter = _delimiter;
      boolean _equals = Objects.equal(delimiter, null);
      if (_equals) {
        delimiter = ";";
      }
      StringBuffer _stringBuffer = new StringBuffer();
      StringBuffer txt = _stringBuffer;
      int _indexOf = _formatString.indexOf("!");
      int _minus = (-1);
      boolean _equals_1 = (_indexOf == _minus);
      if (_equals_1) {
        txt.append(index);
        txt.append(delimiter);
      }
      Object _itemValue = DominoUtils.getItemValue(mailDoc, "DeliveredDate");
      DateTime datetime = ((DateTime) _itemValue);
      char[] formatString = _formatString.toCharArray();
      for (final char _chr : formatString) {
        {
          String chr = Character.toString(_chr);
          boolean _matched = false;
          if (!_matched) {
            if (Objects.equal(chr,"d")) {
              _matched=true;
              String _dateOnly = datetime.getDateOnly();
              txt.append(_dateOnly);
              txt.append(delimiter);
            }
          }
          if (!_matched) {
            if (Objects.equal(chr,"t")) {
              _matched=true;
              String _timeOnly = datetime.getTimeOnly();
              txt.append(_timeOnly);
              txt.append(delimiter);
            }
          }
          if (!_matched) {
            if (Objects.equal(chr,"Z")) {
              _matched=true;
              int _timeZone = datetime.getTimeZone();
              txt.append(_timeZone);
              txt.append(delimiter);
            }
          }
          if (!_matched) {
            if (Objects.equal(chr,"g")) {
              _matched=true;
              String _gMTTime = datetime.getGMTTime();
              txt.append(_gMTTime);
              txt.append(delimiter);
            }
          }
          if (!_matched) {
            if (Objects.equal(chr,"s")) {
              _matched=true;
              String value = mailDoc.getItemValueString("Subject");
              txt.append(value);
              txt.append(delimiter);
            }
          }
          if (!_matched) {
            if (Objects.equal(chr,"f")) {
              _matched=true;
              String value_1 = mailDoc.getItemValueString("From");
              txt.append(value_1);
              txt.append(delimiter);
            }
          }
          if (!_matched) {
            if (Objects.equal(chr,"I")) {
              _matched=true;
              String value_2 = mailDoc.getItemValueString("INetFrom");
              txt.append(value_2);
              txt.append(delimiter);
            }
          }
          if (!_matched) {
            if (Objects.equal(chr,"m")) {
              _matched=true;
              String value_3 = mailDoc.getItemValueString("$Mailer");
              txt.append(value_3);
              txt.append(delimiter);
            }
          }
          if (!_matched) {
            if (Objects.equal(chr,"D")) {
              _matched=true;
              String value_4 = mailDoc.getItemValueString("$MessageID");
              txt.append(value_4);
              txt.append(delimiter);
            }
          }
          if (!_matched) {
            if (Objects.equal(chr,"S")) {
              _matched=true;
              int value_5 = mailDoc.getSize();
              txt.append(value_5);
              txt.append(delimiter);
            }
          }
          if (!_matched) {
            if (Objects.equal(chr,"a")) {
              _matched=true;
              Object _attachmentNames = DominoUtils.getAttachmentNames(mailDoc, session);
              String value_6 = ((String) _attachmentNames);
              boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(value_6);
              if (_isNullOrEmpty) {
                value_6 = "n/a";
              }
              txt.append(value_6);
              txt.append(delimiter);
            }
          }
        }
      }
      String rv = txt.toString();
      int _length = rv.length();
      int _length_1 = delimiter.length();
      int _minus_1 = (_length - _length_1);
      return rv.substring(0, _minus_1);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static Document getLatestMailDocument(final DocumentCollection documentCollection) {
    try {
      Document _xblockexpression = null;
      {
        Document doc = documentCollection.getLastDocument();
        boolean _notEquals = (!Objects.equal(doc, null));
        boolean _while = _notEquals;
        while (_while) {
          {
            Object deliveredDate = DominoUtils.getItemValue(doc, "DeliveredDate");
            boolean _notEquals_1 = (!Objects.equal(deliveredDate, null));
            if (_notEquals_1) {
              return doc;
            }
            Document _prevDocument = documentCollection.getPrevDocument();
            doc = _prevDocument;
          }
          boolean _notEquals_1 = (!Objects.equal(doc, null));
          _while = _notEquals_1;
        }
        _xblockexpression = (null);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static Vector<String> readRecipientFile(final String recipientFile) {
    try {
      Vector<String> _xblockexpression = null;
      {
        File _file = new File(recipientFile);
        File f = _file;
        FileReader _fileReader = new FileReader(f);
        BufferedReader _bufferedReader = new BufferedReader(_fileReader);
        BufferedReader fr = _bufferedReader;
        Vector<Object> _vector = new Vector<Object>();
        Vector<Object> recipients = _vector;
        String line = fr.readLine();
        boolean _notEquals = (!Objects.equal(line, null));
        boolean _while = _notEquals;
        while (_while) {
          {
            recipients.add(line);
            String _readLine = fr.readLine();
            line = _readLine;
          }
          boolean _notEquals_1 = (!Objects.equal(line, null));
          _while = _notEquals_1;
        }
        fr.close();
        String recipientStr = IterableExtensions.join(recipients, ",");
        String[] _recipients = recipientStr.split(",");
        Vector<String> _vector_1 = new Vector<String>();
        Vector<String> recipientVector = _vector_1;
        for (final String recipient : _recipients) {
          String _trim = recipient.trim();
          recipientVector.add(_trim);
        }
        _xblockexpression = (recipientVector);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static String getText(final MIMEEntity mime) {
    try {
      String _xblockexpression = null;
      {
        StringBuffer _stringBuffer = new StringBuffer();
        StringBuffer text = _stringBuffer;
        String _contentType = mime.getContentType();
        boolean _equals = _contentType.equals("multipart");
        if (_equals) {
          MIMEEntity child1 = mime.getFirstChildEntity();
          boolean textAlreadyExtracted = false;
          boolean _notEquals = (!Objects.equal(child1, null));
          boolean _while = _notEquals;
          while (_while) {
            {
              String contentType = child1.getContentType();
              boolean _and = false;
              boolean _equals_1 = Objects.equal(contentType, "text");
              if (!_equals_1) {
                _and = false;
              } else {
                boolean _not = (!textAlreadyExtracted);
                _and = (_equals_1 && _not);
              }
              if (_and) {
                boolean _hasOption = CommandLineArguments.hasOption("all-mime-texts");
                if (_hasOption) {
                  String _plus = ("==MIME part: " + contentType);
                  String _plus_1 = (_plus + "/");
                  String _contentSubType = child1.getContentSubType();
                  String _plus_2 = (_plus_1 + _contentSubType);
                  text.append(_plus_2);
                  text.append(Configuration.LINE_SEPARATOR);
                }
                String _contentAsText = DominoUtils.getContentAsText(child1);
                text.append(_contentAsText);
                boolean _hasOption_1 = CommandLineArguments.hasOption("all-mime-texts");
                boolean _not_1 = (!_hasOption_1);
                if (_not_1) {
                  textAlreadyExtracted = true;
                }
              }
              MIMEEntity child2 = child1.getFirstChildEntity();
              boolean _equals_2 = Objects.equal(child2, null);
              if (_equals_2) {
                MIMEEntity _nextSibling = child1.getNextSibling();
                child2 = _nextSibling;
                boolean _equals_3 = Objects.equal(child2, null);
                if (_equals_3) {
                  MIMEEntity _parentEntity = child1.getParentEntity();
                  child2 = _parentEntity;
                  boolean _notEquals_1 = (!Objects.equal(child2, null));
                  if (_notEquals_1) {
                    MIMEEntity _nextSibling_1 = child2.getNextSibling();
                    child2 = _nextSibling_1;
                  }
                }
              }
              child1 = child2;
            }
            boolean _notEquals_1 = (!Objects.equal(child1, null));
            _while = _notEquals_1;
          }
        } else {
          String contentType = mime.getContentType();
          boolean _equals_1 = Objects.equal(contentType, "text");
          if (_equals_1) {
            String _contentAsText = DominoUtils.getContentAsText(mime);
            text.append(_contentAsText);
          }
        }
        String _string = text.toString();
        _xblockexpression = (_string);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static String getContentAsText(final MIMEEntity mime) {
    try {
      String _xblockexpression = null;
      {
        StringBuffer _stringBuffer = new StringBuffer();
        StringBuffer text = _stringBuffer;
        String subContentType = mime.getContentSubType();
        Logger.log(subContentType);
        boolean _matched = false;
        if (!_matched) {
          if (Objects.equal(subContentType,"html")) {
            _matched=true;
            String txt = mime.getContentAsText();
            boolean _hasOption = CommandLineArguments.hasOption("no-striphtmltags");
            boolean _not = (!_hasOption);
            if (_not) {
              String lowerTxt = txt.toLowerCase();
              boolean _contains = lowerTxt.contains("<head>");
              if (_contains) {
                int start = lowerTxt.indexOf("<head>");
                int end = lowerTxt.indexOf("</head>");
                String _substring = txt.substring(0, start);
                int _length = txt.length();
                String _substring_1 = txt.substring(end, _length);
                String _plus = (_substring + _substring_1);
                txt = _plus;
              }
              String _replaceAll = txt.replaceAll("<[^>]*>", "");
              txt = _replaceAll;
            }
            text.append(txt);
          }
        }
        if (!_matched) {
          String _contentAsText = mime.getContentAsText();
          text.append(_contentAsText);
        }
        text.append(Configuration.LINE_SEPARATOR);
        String _string = text.toString();
        _xblockexpression = (_string);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static String removeMultipleEmptyLines(final String txt) {
    String _xblockexpression = null;
    {
      StringBuffer _stringBuffer = new StringBuffer();
      StringBuffer text = _stringBuffer;
      boolean emptyLastLine = false;
      String[] txtLines = txt.split(Configuration.LINE_SEPARATOR);
      for (final String _line : txtLines) {
        {
          String line = _line.trim();
          boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(line);
          if (_isNullOrEmpty) {
            boolean _not = (!emptyLastLine);
            if (_not) {
              text.append(Configuration.LINE_SEPARATOR);
            }
            emptyLastLine = true;
          } else {
            emptyLastLine = false;
            text.append(line);
            text.append(Configuration.LINE_SEPARATOR);
          }
        }
      }
      String _string = text.toString();
      _xblockexpression = (_string);
    }
    return _xblockexpression;
  }
  
  public static String detachMIMEAttachments(final Document doc, final Session session, final String _detachDir, final boolean getNamesOnly, final String attachmentName) {
    try {
      String _xblockexpression = null;
      {
        String names = "";
        session.setConvertMIME(false);
        MIMEEntity mime = doc.getMIMEEntity();
        boolean _notEquals = (!Objects.equal(mime, null));
        if (_notEquals) {
          String _contentType = mime.getContentType();
          boolean _equals = _contentType.equals("multipart");
          if (_equals) {
            MIMEEntity child1 = mime.getFirstChildEntity();
            boolean _notEquals_1 = (!Objects.equal(child1, null));
            boolean _while = _notEquals_1;
            while (_while) {
              {
                String contentType = child1.getContentType();
                boolean _equals_1 = Objects.equal(contentType, "application");
                if (_equals_1) {
                  String name = null;
                  String _detachAttachment = DominoUtils.detachAttachment(session, child1, _detachDir, getNamesOnly, attachmentName);
                  name = _detachAttachment;
                  boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(name);
                  boolean _not = (!_isNullOrEmpty);
                  if (_not) {
                    String _plus = (names + name);
                    String _plus_1 = (_plus + ",");
                    names = _plus_1;
                  }
                }
                MIMEEntity child2 = child1.getFirstChildEntity();
                boolean _equals_2 = Objects.equal(child2, null);
                if (_equals_2) {
                  MIMEEntity _nextSibling = child1.getNextSibling();
                  child2 = _nextSibling;
                  boolean _equals_3 = Objects.equal(child2, null);
                  if (_equals_3) {
                    MIMEEntity _parentEntity = child1.getParentEntity();
                    child2 = _parentEntity;
                    boolean _notEquals_2 = (!Objects.equal(child2, null));
                    if (_notEquals_2) {
                      MIMEEntity _nextSibling_1 = child2.getNextSibling();
                      child2 = _nextSibling_1;
                    }
                  }
                }
                child1 = child2;
              }
              boolean _notEquals_2 = (!Objects.equal(child1, null));
              _while = _notEquals_2;
            }
          } else {
            String contentType = mime.getContentType();
            boolean _equals_1 = Objects.equal(contentType, "application");
            if (_equals_1) {
              String name = null;
              String _detachAttachment = DominoUtils.detachAttachment(session, mime, _detachDir, getNamesOnly, attachmentName);
              name = _detachAttachment;
            }
          }
        }
        session.setConvertMIME(true);
        boolean _and = false;
        boolean _notEquals_2 = (!Objects.equal(names, null));
        if (!_notEquals_2) {
          _and = false;
        } else {
          boolean _endsWith = names.endsWith(",");
          _and = (_notEquals_2 && _endsWith);
        }
        if (_and) {
          int _length = names.length();
          int _minus = (_length - 1);
          String _substring = names.substring(0, _minus);
          names = _substring;
        }
        _xblockexpression = (names);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static String detachAttachment(final Session session, final MIMEEntity mimeEntity, final String _detachDir, final boolean getNamesOnly, final String attachmentName) {
    try {
      String _xblockexpression = null;
      {
        String name = null;
        String detachDir = _detachDir;
        boolean _equals = Objects.equal(detachDir, null);
        if (_equals) {
          detachDir = "./";
        }
        File _file = new File(detachDir);
        File _dir = _file;
        String dir = _dir.getCanonicalPath();
        boolean _and = false;
        boolean _endsWith = dir.endsWith("/");
        boolean _not = (!_endsWith);
        if (!_not) {
          _and = false;
        } else {
          boolean _endsWith_1 = dir.endsWith("\\");
          boolean _not_1 = (!_endsWith_1);
          _and = (_not && _not_1);
        }
        if (_and) {
          String _plus = (dir + "/");
          dir = _plus;
        }
        mimeEntity.decodeContent();
        DominoUtils.printMIMEHeaders(mimeEntity);
        MIMEHeader header = mimeEntity.getNthHeader("Content-Disposition");
        boolean _notEquals = (!Objects.equal(header, null));
        if (_notEquals) {
          String _headerVal = header.getHeaderVal();
          boolean _equals_1 = Objects.equal(_headerVal, "attachment");
          if (_equals_1) {
            String _paramVal = header.getParamVal("filename");
            name = _paramVal;
            String _replace = name.replace("\"", "");
            String _trim = _replace.trim();
            name = _trim;
            boolean _and_1 = false;
            boolean _not_2 = (!getNamesOnly);
            if (!_not_2) {
              _and_1 = false;
            } else {
              boolean _or = false;
              boolean _equals_2 = Objects.equal(attachmentName, null);
              if (_equals_2) {
                _or = true;
              } else {
                boolean _contains = name.contains(attachmentName);
                _or = (_equals_2 || _contains);
              }
              _and_1 = (_not_2 && _or);
            }
            if (_and_1) {
              Stream stream = session.createStream();
              String _plus_1 = (dir + name);
              boolean _open = stream.open(_plus_1, "binary");
              if (_open) {
                mimeEntity.getContentAsBytes(stream);
                stream.close();
                String _plus_2 = ("   Attachment saved: " + name);
                String _plus_3 = (_plus_2 + ".");
                InputOutput.<String>println(_plus_3);
              } else {
                String _plus_4 = ("   Failed to detach: " + name);
                String _plus_5 = (_plus_4 + ".");
                InputOutput.<String>println(_plus_5);
              }
            }
          }
        }
        _xblockexpression = (name);
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static void printMIMEHeaders(final MIMEEntity mimeEntity) {
    try {
      Vector headers = mimeEntity.getHeaderObjects();
      String _string = mimeEntity.toString();
      String _plus = ("=== MIME Entity: " + _string);
      Logger.log(_plus);
      for (final Object _header : headers) {
        {
          MIMEHeader header = ((MIMEHeader) _header);
          String _headerName = header.getHeaderName();
          Logger.log(_headerName);
          String _headerVal = header.getHeaderVal();
          String _plus_1 = ("  value : " + _headerVal);
          Logger.log(_plus_1);
          String _headerValAndParams = header.getHeaderValAndParams();
          String _plus_2 = ("  value and params: " + _headerValAndParams);
          Logger.log(_plus_2);
        }
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static void printFields(final Document document) {
    try {
      Vector items = document.getItems();
      HashMap<String,String> columnFields = CollectionLiterals.<String, String>newHashMap();
      for (final Object _item : items) {
        {
          Item item = ((Item) _item);
          String name = item.getName();
          int _type = item.getType();
          Class<? extends Item> _class = item.getClass();
          String className = _class.getName();
          String type = "";
          boolean _matched = false;
          if (!_matched) {
            if (Objects.equal(_type,Item.ACTIONCD)) {
              _matched=true;
              type = "ACTIONCD";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.ASSISTANTINFO)) {
              _matched=true;
              type = "ASSISTANTINFO";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.ATTACHMENT)) {
              _matched=true;
              type = "ATTACHMENT (file attachment)";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.AUTHORS)) {
              _matched=true;
              type = "AUTHORS";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.COLLATION)) {
              _matched=true;
              type = "COLLATION";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.DATETIMES)) {
              _matched=true;
              type = "DATETIMES (date-time or range of date-time values)";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.EMBEDDEDOBJECT)) {
              _matched=true;
              type = "EMBEDDEDOBJECT";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.ERRORITEM)) {
              _matched=true;
              type = "ERRORITEM (error occurred while getting type)";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.FORMULA)) {
              _matched=true;
              type = "FORMULA (Domino formula)";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.HTML)) {
              _matched=true;
              type = "HTML (HTML source text)";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.ICON)) {
              _matched=true;
              type = "ICON";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.LSOBJECT)) {
              _matched=true;
              type = "LSOBJECT";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.MIME_PART)) {
              _matched=true;
              type = "MIME_PART";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.NAMES)) {
              _matched=true;
              type = "NAMES";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.NOTELINKS)) {
              _matched=true;
              type = "NOTELINKS (link to a database, view, or document)";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.NOTEREFS)) {
              _matched=true;
              type = "NOTEREFS (reference to the parent document)";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.NUMBERS)) {
              _matched=true;
              type = "NUMBERS (number or number list)";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.OTHEROBJECT)) {
              _matched=true;
              type = "OTHEROBJECT";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.QUERYCD)) {
              _matched=true;
              type = "QUERYCD";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.READERS)) {
              _matched=true;
              type = "READERS";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.RFC822TEXT)) {
              _matched=true;
              type = "RFC822TEXT";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.RICHTEXT)) {
              _matched=true;
              type = "RICHTEXT";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.SIGNATURE)) {
              _matched=true;
              type = "SIGNATURE";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.TEXT)) {
              _matched=true;
              type = "TEXT (text or text list)";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.UNAVAILABLE)) {
              _matched=true;
              type = "UNAVAILABLE";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.UNKNOWN)) {
              _matched=true;
              type = "UNKNOWN";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.USERDATA)) {
              _matched=true;
              type = "USERDATA";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.USERID)) {
              _matched=true;
              type = "USERID";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.VIEWMAPDATA)) {
              _matched=true;
              type = "VIEWMAPDATA";
            }
          }
          if (!_matched) {
            if (Objects.equal(_type,Item.VIEWMAPLAYOUT)) {
              _matched=true;
              type = "VIEWMAPLAYOUT";
            }
          }
          if (!_matched) {
            type = "undetermined";
          }
          String _plus = (type + ",");
          String _plus_1 = (_plus + className);
          columnFields.put(name, _plus_1);
        }
      }
      Set<String> _keySet = columnFields.keySet();
      List<String> sortedNames = IterableExtensions.<String>sort(_keySet);
      int index = 1;
      boolean printIndex = false;
      int _size = sortedNames.size();
      String _plus = ("Total fields: " + Integer.valueOf(_size));
      InputOutput.<String>println(_plus);
      ArrayList<String> list = CollectionLiterals.<String>newArrayList();
      list.add("NAME");
      list.add("TYPE");
      Output.prettyPrintln(list, 30, 0);
      for (final String name : sortedNames) {
        {
          ArrayList<String> _newArrayList = CollectionLiterals.<String>newArrayList();
          list = _newArrayList;
          String indexS = "";
          if (printIndex) {
            boolean _lessThan = (index < 10);
            if (_lessThan) {
              String _plus_1 = (Integer.valueOf(index) + " : ");
              indexS = _plus_1;
            } else {
              String _plus_2 = (Integer.valueOf(index) + ": ");
              indexS = _plus_2;
            }
          }
          String _plus_3 = (indexS + name);
          list.add(_plus_3);
          String _get = columnFields.get(name);
          String[] typeAndClass = _get.split(",");
          String _get_1 = typeAndClass[0];
          list.add(_get_1);
          Output.prettyPrintln(list, 30, 0);
          int _plus_4 = (index + 1);
          index = _plus_4;
        }
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static void printFieldValue(final Document document, final String fieldName) {
    try {
      boolean _hasItem = document.hasItem(fieldName);
      boolean _not = (!_hasItem);
      if (_not) {
        String _plus = ("Field " + fieldName);
        String _plus_1 = (_plus + " does not exist.");
        InputOutput.<String>println(_plus_1);
        return;
      }
      String _plus_2 = ("Field " + fieldName);
      String _plus_3 = (_plus_2 + ":");
      InputOutput.<String>println(_plus_3);
      Vector itemValues = document.getItemValue(fieldName);
      int _size = itemValues.size();
      boolean _equals = (_size == 0);
      if (_equals) {
        InputOutput.<String>println("  N/A");
      }
      for (final Object value : itemValues) {
        String _plus_4 = ("  " + value);
        InputOutput.<String>println(_plus_4);
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
